
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "Serial2Wifi_SPI_Client.h"

int main(int argc, char* argv[])
{
	if( argc < 4 )
	{
		printf("USAGE: SPI_Benchmark PacketSize IPAddress Port\n");
		return(-1);
	}

	int txSize = atoi(argv[1]);

	if( txSize < 0 || txSize > 1460 )
	{
		printf("ERROR: Invalid packet size [1-1460]\n");
		return(-1);
	}

	int iteration_count = 100;

	int port = atoi(argv[3]);

	if( port < 0 )
	{
		printf("ERROR: Invalid port number\n");
		return(-1);
	}


///////////////////////////////////////////////////////////////////////////////

	printf("Serial2Wifi SPI Benchmark v1.0\n\n");
	printf("Initializing SPI\n\n");

	Serial2Wifi_SPI_Client spi;

	try
	{
		spi.initialize(115200);
	}
	catch( const char *e )
	{
		printf("%s\n",e);
		return(-1);
	}


///////////////////////////////////////////////////////////////////////////////

	char OutputBuffer[1500];
	char InputBuffer[1500];

//	char *InputBuffer = (char *)malloc(sizeof(char) * 1000);

	int r = 0;

	// Rest the board
	printf("ZR\n");
	spi.sendCommandBlocking("ZR\r",InputBuffer);

	// Set SSID
	printf("C1=SSID\n");
	spi.sendCommandBlocking("C1=SSID\r", InputBuffer);

	// Set security to open
	printf("C3=0\n");
	r = spi.sendCommandBlocking("C3=0\r",InputBuffer);

	// Use DHCP
	printf("C4=1\n");
	spi.sendCommandBlocking("C4=1\r",InputBuffer);

	// Join the network
	printf("C0\n");
	r = spi.sendCommandBlocking("C0\r",InputBuffer);

	// Check to make sure join was successful
	if( r == 0 )
	{
		printf("ERROR: Failed to join network\n");
		return(-1);
	}

	// Use TCPIP
	printf("P1=0\n");
	spi.sendCommandBlocking("P1=0\r",InputBuffer);

	// Set the server IP address
	printf("P3=%s\n",argv[2]);
	sprintf(OutputBuffer, "P3=%s\r", argv[2]);
	spi.sendCommandBlocking(OutputBuffer, InputBuffer);

	// Set the server port
	printf("P4=%d\n",port);
	sprintf(OutputBuffer, "P4=%d\r",port);
	spi.sendCommandBlocking("P4=8022\r", InputBuffer);

	// Set the packet size
	printf("S1=%d\n", txSize);
	sprintf(OutputBuffer, "S1=%d\r", txSize);
	spi.sendCommandBlocking(OutputBuffer, InputBuffer);

	// Start the client
	printf("P6=1\n");
	r = spi.sendCommandBlocking("P6=1\r", InputBuffer);

	if( r == 0 )
	{
		printf("ERROR: Failed to connect to server\n");
		return(-1);
	}

	char value = 97 + (txSize%26);

	char outputStr[1500];
	for( int i = 0; i < txSize; i++ )
		outputStr[i] = value;
	for( int i = txSize; i < 1500; i++ )
		outputStr[i] = '\0';
	sprintf(OutputBuffer, "S0\r%s\r", outputStr);

	struct timespec start, stop;
	unsigned long long int diff = 0;
	unsigned long long int diffTemp = 0;

	int valid_iter = iteration_count;
	for (int iter = 0; iter < iteration_count; iter++)
	{
		// Time the sending of a packet
		clock_gettime(CLOCK_REALTIME, &start);
		spi.sendCommandBlocking(OutputBuffer, InputBuffer);
		clock_gettime(CLOCK_REALTIME, &stop);

		diffTemp = stop.tv_nsec - start.tv_nsec;

		if( start.tv_nsec > stop.tv_nsec )
			valid_iter--;
		else
			diff = diff + diffTemp;
	}

	if( diff < 0 )
		diff += 1000000000;

	printf("\n\n");
	printf("packet size [bytes]    = %d\n", txSize);
	printf("iteration count        = %d\n", iteration_count);
	printf("average exec time [ns] = %f\n",((float)diff)/valid_iter);

	return 0;
}

